import six
import json

from prompt_toolkit.application import get_app
from prompt_toolkit.filters import to_filter
from prompt_toolkit.mouse_events import MouseEventType
from prompt_toolkit.key_binding.key_bindings import KeyBindings
from prompt_toolkit.layout.containers import Window
from prompt_toolkit.layout.controls import FormattedTextControl

from freud.model import db
from freud.ui.text_buffers import summary_buffer


class ButtonManager:
    """
    Button logic flows through here. Button Manager saves the current button,
    previous button, list of buttons, and provides a click handler.
    """

    current_button = None
    prev_button = None
    buttons = None

    def __init__(self, name):
        self.name = name

    def click_handler(self):

        result = db.fetch_one(name=self.name)

        output = {
            'name': result.name,
            'method': result.method,
            'url': result.url
        }

        if result.headers:
            output.update({
                'headers': json.loads(result.headers)
            })
        if result.body:
            try:
                body = json.loads(result.body)
            except json.decoder.JSONDecodeError:
                body = str(result.body).splitlines()

            output.update({
                'body': body
            })
        if result.auth:
            auth = json.loads(result.auth)
            output.update({
                'auth': {
                    'type': auth['type'],
                    'user': auth['user']
                }
            })

        summary_buffer.read_only = to_filter(False)
        summary_buffer.text = json.dumps(output, indent=2)
        summary_buffer.read_only = to_filter(True)

        type(self).current_button = self.name

        app = get_app()
        type(self).prev_button = app.layout.current_window

    @classmethod
    def update_buttons(cls, app):

        windows = [w for w in app.layout.find_all_windows()]
        cls.buttons = []
        for window in windows:
            if 'CustomButton' in str(window):
                cls.buttons.append(window)

        if cls.prev_button is None and len(cls.buttons) > 0:
            cls.prev_button = cls.buttons[0]

        return cls.buttons


class SortOrder:
    """ Saves the sort order for reference by application """

    sort_by = None
    order = None


class CustomButton:
    """
    Taken from Python Prompt Toolkit's Button class for customization

    Clickable button.
    :param text: The caption for the button.
    :param handler: `None` or callable. Called when the button is clicked.
    :param width: Width of the button.
    """

    def __init__(self, text, handler=None):
        assert isinstance(text, six.text_type)
        assert handler is None or callable(handler)

        self.text = text
        self.handler = handler
        self.control = FormattedTextControl(
            self._get_text_fragments,
            key_bindings=self._get_key_bindings(),
            show_cursor=False,
            focusable=True)

        def get_style():
            if get_app().layout.has_focus(self):
                return 'class:button.focused'

            return 'class:button'

        self.window = Window(
            self.control,
            height=1,
            style=get_style,
            dont_extend_width=True,
            dont_extend_height=True)

    def _get_text_fragments(self):
        text = self.text

        def handler(mouse_event):
            if mouse_event.event_type == MouseEventType.MOUSE_UP:
                self.handler()

        return [
            ('class:button.arrow', '', handler),
            ('class:button.text', text, handler),
            ('class:button.arrow', '', handler),
        ]

    def _get_key_bindings(self):
        kb = KeyBindings()

        @kb.add(' ')
        @kb.add('enter')
        def _(event):
            if self.handler is not None:
                self.handler()

        return kb

    def __pt_container__(self):
        return self.window


def on_startup(app):
    """ Run from __main__ after render """

    ButtonManager.update_buttons(app)
    if not summary_buffer.text:
        # When starting app, select first server if none selected
        select_item(app)


class SingleClick:
    """ Provides mouse click key """

    def __init__(self):
        self.event_type = MouseEventType.MOUSE_UP


def select_item(event):
    """ Simulate mouse click """

    # Don't try to select button if there are none
    if len(ButtonManager.buttons) > 0:

        try:

            event.app.layout.current_window.content.text()[1][2](SingleClick())

        # If app is passed in, rather than event
        except AttributeError:

            event.layout.current_window.content.text()[1][2](SingleClick())



import requests
import json
import xml

from prompt_toolkit.lexers import PygmentsLexer
from pygments.lexers import HtmlLexer, JsonLexer, XmlLexer

from freud.ui.text_buffers import response_box
from freud import INDENTATION


def response_handler(response):
    # Format content for header_buffer
    #
    status_code_lookup = requests.status_codes._codes
    status_description = status_code_lookup[response.status_code][0]
    status_description = status_description[0].upper(
    ) + status_description[1:]

    seconds = response.elapsed.total_seconds()
    elapsed = str('{}ms'.format(round(seconds * 1000, 2)))

    # if elapsed time is 1 second or greater, display seconds instead of ms
    if (len(elapsed.split('.')[0])) > 3:
        elapsed = str('{}s'.format(round(seconds, 2)))

    status_code = ['{} {}'.format(
        response.status_code, status_description)]

    headers = response.headers

    headers_string = ''
    for k, v in headers.items():
        headers_string += '{}: {}\n'.format(k, v)
    headers_string = headers_string.rstrip('\n')

    headers = '{} | {}\n{}'.format(status_code, elapsed, headers_string)

    response_body = content_type_handler(response)

    # Format content for response_buffer
    #
    return headers, response_body


def content_type_handler(response):

    content_type = response.headers.get('Content-Type')
    response_body = None
    if content_type:

        if content_type.startswith('application/json'):
            try:
                response_body = json.dumps(response.json(), indent=INDENTATION)
                response_box.buffer_control.lexer = PygmentsLexer(
                    JsonLexer)
            except json.decoder.JSONDecodeError as e:
                response_body = 'JSON Error: {}\n\n'.format(str(e))
                response_body += response.text

        elif content_type.startswith('text/html'):
            response_body = response.text
            response_box.buffer_control.lexer = PygmentsLexer(
                HtmlLexer)

        elif content_type.startswith('text/xml'):
            try:
                xml_data = xml.dom.minidom.parseString(response.text)
                response_body = xml_data.toprettyxml(indent=' ' * 2)
                response_box.buffer_control.lexer = PygmentsLexer(
                    XmlLexer)
            except xml.parsers.expat.ExpatError as e:
                response_body = 'XML Error: {}\n\n'.format(str(e))
                response_body += response.text

    if not response_body:
        response_body = response.text

    return response_body



import asyncio

from prompt_toolkit.filters.utils import to_filter

from freud.ui.dialog import LoadingDialog, ErrorDialog
from freud.utils import ButtonManager
from freud.ui.text_buffers import (
    response_buffer, header_buffer)
from freud.api.request import request_handler


def send_request(event):
    """ Prepare and send request, then display the response """

    name = ButtonManager.current_button

    # Display loading dialog
    loading_dialog = LoadingDialog(
        event=event,
        title='Loading',
        text='Connecting to {}...'.format(name)
    )

    async def send_request_async():
        # Cede control of event loop. See:
        # github.com/python/asyncio/issues/284ield
        await asyncio.sleep(0)

        # Build and send request to server
        result = await request_handler(name)

        loading_dialog.close_dialog()

        errors = result.get('errors')

        if errors:
            error_type, error_value = list(errors.items())[0]

            header_buffer.read_only = to_filter(False)
            header_buffer.text = error_type
            header_buffer.read_only = to_filter(True)

            response_buffer.read_only = to_filter(False)
            response_buffer.text = ''
            response_buffer.read_only = to_filter(True)

            # LoadingDialog is not removed if called a second time. Invalidate
            # method sends a repaint trigger
            event.app.invalidate()

            return ErrorDialog(event, title=error_type,
                               text=error_value)

        headers, response_body = result.get('response')

        header_buffer.read_only = to_filter(False)
        header_buffer.text = headers
        header_buffer.read_only = to_filter(True)

        response_buffer.read_only = to_filter(False)
        response_buffer.text = response_body
        response_buffer.read_only = to_filter(True)

    asyncio.ensure_future(send_request_async())



import pytest
import json
import os
import sys


def pytest_configure():
    """ Freud checks for this upon startup. If true, creates a new test.db """

    sys._called_from_test = True


def pytest_unconfigure():
    from freud import DB_FILE
    from freud.model import basedir

    del sys._called_from_test
    os.remove(os.path.join(basedir, DB_FILE))


@pytest.fixture(scope='class')
def db_conn():

    from freud.model import Db
    db = Db()

    yield db


@pytest.fixture(scope='function')
def db(db_conn):

    db_conn.delete_all()

    return db_conn


@pytest.fixture(scope='function')
def db_dummy_data(db):
    db.add_one(
        {'name': 'alice', 'url': 'alice.com', 'method': 'get'})

    db.add_one(
        {'name': 'bob', 'url': 'bob.com', 'method': 'get'})

    return


@pytest.fixture(scope='class')
def db_request_data(httpbin, db_conn):

    # Get request
    #
    db_conn.add_one({
        'name': 'httpbin-get',
        'url': '{}/get'.format(httpbin.url),
        'method': 'GET'
    })

    # Delete request
    #
    db_conn.add_one({
        'name': 'httpbin-delete',
        'url': '{}/delete'.format(httpbin.url),
        'method': 'DELETE',
    })

    # Patch request
    #
    db_conn.add_one({
        'name': 'httpbin-patch',
        'url': '{}/patch'.format(httpbin.url),
        'method': 'PATCH',
    })

    # Put request
    #
    db_conn.add_one({
        'name': 'httpbin-put',
        'url': '{}/put'.format(httpbin.url),
        'method': 'PUT',
    })

    # Post request
    #
    db_conn.add_one({
        'name': 'httpbin-post',
        'url': '{}/post'.format(httpbin.url),
        'method': 'POST',
        'body': json.dumps({'name': 'alice'}),
        'headers': json.dumps({'Content-Type': 'application/json'})
    })

    # Post form
    #
    db_conn.add_one({
        'name': 'httpbin-post-form',
        'url': '{}/post'.format(httpbin.url),
        'method': 'POST',
        'headers': json.dumps(
            {'content-type': 'application/x-www-form-urlencoded'}
        ),
        'body': 'name=alice&email=alice@email.com'
    })

    # Basic auth
    #
    db_conn.add_one({
        'name': 'httpbin-basic-auth-200',
        'url': '{}/basic-auth/alice/password'.format(httpbin.url),
        'method': 'GET',
        'auth': json.dumps({
            'user': 'alice',
            'password': 'password',
            'type': 'basic'
        })
    })
    db_conn.add_one({
        'name': 'httpbin-basic-auth-401',
        'url': '{}/basic-auth/alice/password'.format(httpbin.url),
        'method': 'GET',
        'auth': json.dumps({
            'user': 'eve',
            'password': 'bad_password',
            'type': 'basic'
        })
    })
    db_conn.add_one({
        'name': 'httpbin-digest-auth-200',
        'url': '{}/digest-auth/auth/alice/password'.format(httpbin.url),
        'method': 'GET',
        'auth': json.dumps({
            'user': 'alice',
            'password': 'password',
            'type': 'digest'
        })
    })
    db_conn.add_one({
        'name': 'httpbin-digest-auth-401',
        'url': '{}/digest-auth/auth/alice/password'.format(httpbin.url),
        'method': 'GET',
        'auth': json.dumps({
            'user': 'eve',
            'password': 'bad_password',
            'type': 'digest'
        })
    })

    return



from prompt_toolkit.layout.dimension import D
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.document import Document
from prompt_toolkit.layout import Layout
from prompt_toolkit.widgets import TextArea, Dialog, Label, Button
from prompt_toolkit.layout.containers import HSplit, Window
from prompt_toolkit.layout import Float

from freud.ui.root_container import root_container
from freud.model import db
from freud.utils import ButtonManager, select_item
from freud.ui.dialog import ErrorDialog


class ServerDialog:
    def __init__(self, event, create_server=False):

        def ok_handler():

            # Simplistic check to see if user forgot to enter something for url
            url = len(self.url.text) > len('https://')

            if not all([self.name.text, url, self.method.text]):
                return ErrorDialog(event, title='Input Error',
                                   text='Name, Url, and Method are required.')

            result = server_to_db()

            if result.get('success'):

                root_container.floats.pop()

                # Rather than inserting a new button into, e.g.,
                # hsplit.children, we recreate the layout since we have to
                # pay attention to sort order here
                event.app.layout = Layout(root_container.create())

                # Find the button in the redrawn layout; then focus on it
                buttons = ButtonManager.update_buttons(event.app)
                for button in buttons:
                    if self.name.text == button.content.text()[1][1].strip():
                        event.app.layout.focus(button)
                        break

                select_item(event)

            else:
                # Add/update server returned an error
                ErrorDialog(event, title='Add/edit server error',
                            text=str(result.get('errors')))

        def cancel_handler():
            root_container.floats.pop()
            if ButtonManager.prev_button:
                event.app.layout.focus(ButtonManager.prev_button)

        def server_to_db():
            if self.rowid:
                # Updating an existing server
                return db.update_one(
                    rowid=self.rowid,
                    values={
                        'name': self.name.text,
                        'url': self.url.text,
                        'method': self.method.text
                    })
            else:
                # Creating a new server
                return db.add_one(
                    values={
                        'name': self.name.text,
                        'url': self.url.text,
                        'method': self.method.text
                    })

        # Get data about server currently editing
        if create_server:
            title = 'New Server'
            self.rowid = None
            name = ''
            url = 'https://'
            method = ''
        else:
            title = 'Edit Server'
            self.name = ButtonManager.current_button
            result = db.fetch_one(name=self.name)

            self.rowid = result.rowid
            name = result.name
            url = result.url
            method = result.method

        # Dialog configuration
        ok_button = Button(text='OK', handler=ok_handler)
        cancel_button = Button(text='Cancel', handler=cancel_handler)

        methods = [
            'GET',
            'POST',
            'PUT',
            'HEAD',
            'DELETE',
            'CONNECT',
            'OPTIONS',
            'TRACE',
            'PATCH'
        ]
        method_completer = WordCompleter(methods,
                                         ignore_case=True)

        local_kb = KeyBindings()

        @local_kb.add('enter')
        def cancel_completion(event):
            buff = event.app.current_buffer
            buff.complete_state = None
            event.app.layout.focus_next()

        self.name = TextArea(
            multiline=False,
            text=name
        )
        self.name.buffer.document = Document(name, len(name))

        self.url = TextArea(
            multiline=False,
            text=url
        )
        self.url.buffer.document = Document(url, len(url))

        self.method = TextArea(
            multiline=False,
            completer=method_completer,
            text=method)
        self.method.buffer.document = Document(method, len(method))

        self.dialog = Dialog(
            title=title,
            body=HSplit([
                Label(text='Server name:\n'),
                self.name,
                Window(height=1, char=' '),
                Label(text='Url:\n'),
                self.url,
                Window(height=1, char=' '),
                Label(text='Method:\n'),
                self.method,
                Window(height=1, char=' ')
            ],
                key_bindings=local_kb
            ),
            buttons=[ok_button, cancel_button],
            width=D(preferred=80),
            with_background=True
        )

        self.current_button = event.app.layout.current_window

        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)



import json
from prompt_toolkit.layout.layout import Layout
from .utils import SingleClick
from freud.utils import ButtonManager
from freud.ui.server_container import servers
from freud.ui.root_container import root_container
from freud.key_bindings import server_kb


class TestChangingLayout:

    @staticmethod
    def _get_windows(root_container):
        root_container = root_container.create()
        layout = Layout(root_container)
        all_windows = layout.find_all_windows()
        windows = []
        for window in all_windows:
            windows.append(window)

        return windows

    @staticmethod
    def _get_all_windows(layout):
        return [w for w in layout.find_all_windows()]

    def test_check_layout(self, db_dummy_data):

        windows = self._get_windows(root_container)

        search_buffer = windows[0]
        title_window = windows[1]
        dummy_one = windows[4]
        button_one = windows[5]
        button_two = windows[6]
        dummy_two = windows[7]
        header_buffer = windows[10]
        response_buffer = windows[12]
        summary_buffer = windows[14]

        assert 'SearchBuffer' in str(search_buffer)
        assert 'Freud' in str(title_window)
        assert 'CustomButton' not in str(dummy_one)
        assert 'CustomButton' in str(button_one)
        assert 'CustomButton' in str(button_two)
        assert 'CustomButton' not in str(dummy_two)
        assert 'header_buffer' in str(header_buffer)
        assert 'response_buffer' in str(response_buffer)
        assert 'summary_buffer' in str(summary_buffer)

    def test_update_response_box(self, db_dummy_data):

        # Create layout
        layout = Layout(root_container.create(),
                        focused_element=servers.content)

        summary_buffer = layout.get_buffer_by_name('summary_buffer')

        # Simulate mouse click on first server
        layout.current_window.content.text()[1][2](SingleClick())
        summary_buffer_before = json.loads(summary_buffer.text)

        # Get buttons for focus
        app = App(layout)
        ButtonManager.update_buttons(app)

        # Focus and simulate click on next server
        layout.focus(ButtonManager.buttons[1])
        layout.current_window.content.text()[1][2](SingleClick())
        summary_buffer_after = json.loads(summary_buffer.text)

        assert summary_buffer_before['name'] == 'alice'
        assert summary_buffer_after['name'] == 'bob'

    def test_add_server(self, db, db_dummy_data):
        db.add_one({'name': 'john', 'url': 'john.com', 'method': 'get'})

        windows = self._get_windows(root_container)

        dummy_one = windows[4]
        button_one = windows[5]
        button_two = windows[6]
        button_three = windows[7]
        dummy_two = windows[8]

        assert 'CustomButton' not in str(dummy_one)
        assert 'CustomButton' in str(button_one)
        assert 'CustomButton' in str(button_two)
        assert 'CustomButton' in str(button_three)
        assert 'CustomButton' not in str(dummy_two)

    def test_delete_server(
            self, db, db_dummy_data):

        # Get handler for delete server key binding
        for key in server_kb.bindings:
            if 'rm_server' in str(key):
                delete_handler = key.handler

        # Create layout
        root = root_container.create()
        layout = Layout(root,
                        focused_element=servers.content)
        windows = self._get_all_windows(layout)

        event = Event(layout)

        # Simulate mouse click on first server
        layout.current_window.content.text()[1][2](SingleClick())

        # Simulate pressing key binding for deleting a server
        delete_handler(event)

        # When a server is deleted in the app we get a delete confirmation
        # dialog, which is stored in root.floats. Check for that
        # here.
        windows = self._get_all_windows(layout)
        floats = root.floats

        assert len(floats) == 2

        # Simulate pressing the OK button in the delete confirmation dialog
        windows[34].content.text()[1][2](SingleClick())

        assert len(floats) == 1

        windows = self._get_all_windows(layout)

        dummy_one = str(windows[4])
        button_one = str(windows[5])
        dummy_two = str(windows[6])

        assert 'Button' not in dummy_one
        assert 'Button' in button_one
        assert 'Button' not in dummy_two

    def test_sort_dialog(self, db_dummy_data):

        for key in server_kb.bindings:
            if 'sort' in str(key):
                sort_handler = key.handler

        # Create layout
        root = root_container.create()
        layout = Layout(root,
                        focused_element=servers.content)
        windows = self._get_all_windows(layout)

        assert len(root_container.floats) == 1

        event = Event(layout)
        sort_handler(event)

        assert len(root_container.floats) == 2

        windows = self._get_all_windows(layout)
        windows[33].content.text()[1][2](SingleClick())

        assert len(root_container.floats) == 1


class Event:
    def __init__(self, layout):
        self.app = App(layout)
        self.event = self.app


class App:
    def __init__(self, layout):
        self.layout = layout



from prompt_toolkit.layout.dimension import D
from prompt_toolkit.layout import Float
from prompt_toolkit.layout import Layout
from prompt_toolkit.widgets import Button, Dialog, RadioList
from prompt_toolkit.key_binding import KeyBindings

from freud.utils import ButtonManager, SortOrder
from freud.ui.root_container import root_container
from freud.ui.server_container import servers


class SortDialog:
    def __init__(self, event):
        kb = KeyBindings()

        def ok_handler():

            sort_by = self.radio_list.current_value['sort_by']
            order = self.radio_list.current_value['order']

            SortOrder.sort_by = sort_by
            SortOrder.order = order

            root_container.floats.pop()

            event.app.layout = Layout(root_container.create(),
                                      focused_element=servers.content
                                      )
            ButtonManager.prev_button = event.app.layout.current_window

        def cancel_handler():
            root_container.floats.pop()
            root_container.float_container.key_bindings = None
            event.app.layout.focus(ButtonManager.prev_button)

        ok_button = Button(text='OK', handler=ok_handler)
        cancel_button = Button(text='Cancel', handler=cancel_handler)

        self.radio_list = RadioList(values=[
            ({'sort_by': 'name', 'order': 'asc'}, 'Name'),
            ({'sort_by': 'name', 'order': 'desc'}, 'Name - Desc'),
            ({'sort_by': 'timestamp', 'order': 'asc'}, 'Time Added'),
            ({'sort_by': 'timestamp', 'order': 'desc'}, 'Time Added - Desc')
        ])

        kb = self.radio_list.control.key_bindings

        @kb.add('j')
        def down(event):
            self.radio_list._selected_index = min(
                len(self.radio_list.values) - 1,
                self.radio_list._selected_index + 1
            )

        @kb.add('k')
        def up(event):
            self.radio_list._selected_index = max(
                0, self.radio_list._selected_index - 1)

        @kb.add('g', 'g')
        def top(event):
            self.radio_list._selected_index = 0

        @kb.add('G')
        def bottom(event):
            self.radio_list._selected_index = len(self.radio_list.values) - 1

        self.dialog = Dialog(
            title='Sort',
            body=self.radio_list,
            buttons=[ok_button, cancel_button],
            width=D(preferred=80),
            with_background=True,
            modal=True)

        root_container.float_container.key_bindings = kb

        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)



import json

from prompt_toolkit.layout.dimension import D
from prompt_toolkit.document import Document
from prompt_toolkit.layout import Float
from prompt_toolkit.widgets import Button, TextArea, Dialog, Label
from prompt_toolkit.layout.containers import HSplit, Window
from prompt_toolkit.widgets import RadioList
from prompt_toolkit.key_binding import KeyBindings

from freud.ui.root_container import root_container
from freud.ui.dialog import ErrorDialog
from freud.model import db
from freud.utils import ButtonManager, select_item


class DeleteConfirmation:
    """ Confirm deletion of authentication information """

    def __init__(self, event, dialog):

        def ok_handler():
            root_container.floats.pop()
            db.update_one(values={'name': name,
                                  'auth': None})
            event.app.layout.focus(ButtonManager.prev_button)
            select_item(event)

        def cancel_handler():
            root_container.floats.pop()
            root_container.floats.append(self.auth_float)
            event.app.layout.focus(dialog)

        ok_button = Button(text='OK', handler=ok_handler)
        cancel_button = Button(text='Cancel', handler=cancel_handler)

        name = ButtonManager.current_button

        self.dialog = Dialog(
            title='Delete confirmation',
            body=Label(
                text='Are you sure you want to delete authentication for {}?'
                .format(name)
            ),
            buttons=[cancel_button, ok_button],
            width=D(preferred=80),
            with_background=True
        )

        self.auth_float = root_container.floats.pop()
        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)


class AuthSelector:
    """ Provide selection of authentication types. If the server already has
    authentication, skip selecting and open the respective dialog box """

    def __init__(self, event):

        def dialog_opener(authtype, auth={}):

            if authtype == 'basic':
                BasicAuthDialog(event, auth)
            elif authtype == 'digest':
                DigestAuthDialog(event, auth)

        def ok_handler():

            root_container.floats.pop()

            authtype = self.radio_list.current_value['authtype']
            dialog_opener(authtype)

        def cancel_handler():
            root_container.floats.pop()
            root_container.float_container.key_bindings = None

            event.app.layout.focus(ButtonManager.prev_button)

        kb = KeyBindings()

        server = db.fetch_one(name=ButtonManager.current_button)

        if server.auth:

            auth = json.loads(server.auth)
            auth_type = auth.get('type')
            dialog_opener(auth_type, auth=auth)

        else:

            ok_button = Button(text='OK', handler=ok_handler)
            cancel_button = Button(text='Cancel', handler=cancel_handler)

            self.radio_list = RadioList(values=[
                ({'authtype': 'basic'}, 'Basic'),
                ({'authtype': 'digest'}, 'Digest')
            ])

            kb = self.radio_list.control.key_bindings

            @kb.add('j')
            def down(event):
                self.radio_list._selected_index = min(
                    len(self.radio_list.values) - 1,
                    self.radio_list._selected_index + 1
                )

            @kb.add('k')
            def up(event):
                self.radio_list._selected_index = max(
                    0, self.radio_list._selected_index - 1)

            @kb.add('g', 'g')
            def top(event):
                self.radio_list._selected_index = 0

            @kb.add('G')
            def bottom(event):
                self.radio_list._selected_index = len(
                    self.radio_list.values) - 1

            self.dialog = Dialog(
                title='Select auth type',
                body=self.radio_list,
                buttons=[ok_button, cancel_button],
                width=D(preferred=80),
                with_background=True,
                modal=True)

            root_container.float_container.key_bindings = kb

            root_container.floats.append(Float(self.dialog))
            event.app.layout.focus(self.dialog)


class AuthDialog:
    """ Parent dialog for authentication """

    def __init__(self):

        self.name = ButtonManager.current_button

        self.ok_button = Button(text='OK', handler=self.ok_handler)

        self.cancel_button = Button(
            text='Cancel', handler=self.cancel_handler)

        self.delete_button = Button(
            text='Delete', handler=self.delete_handler)

        self.authuser = TextArea(
            multiline=False
        )

        self.authpass_one = TextArea(
            multiline=False,
            password=True
        )

        self.authpass_two = TextArea(
            multiline=False,
            password=True
        )

    def ok_handler(self):
        authuser = self.authuser.text
        authpass = self.authpass_one.text

        all_fields = all([authuser, authpass])
        empty_fields = not any([authuser, authpass])

        if authpass != self.authpass_two.text:
            ErrorDialog(self.event, title='Password Error',
                        text='Passwords do not match, please try again')

        elif all_fields or empty_fields:

            if all_fields:
                result = db.update_one({
                    'name': self.name,
                    'auth': json.dumps({
                        'user': authuser,
                        'password': authpass,
                        'type': self.authtype
                    })
                })

            elif empty_fields:
                result = db.update_one(values={'name': self.name,
                                               'auth': None})

            if result.get('success'):
                root_container.floats.pop()
                self.event.app.layout.focus(ButtonManager.prev_button)
                select_item(self.event)

            else:
                ErrorDialog(self.event, title='Add/edit server error',
                            text=result.get('errors'))

        else:
            ErrorDialog(self.event, title='Input Error',
                        text='Missing one or more fields.')

    def delete_handler(self):
        DeleteConfirmation(self.event, self.dialog)

    def cancel_handler(self):
        root_container.floats.pop()
        self.event.app.layout.focus(ButtonManager.prev_button)


class BasicAuthDialog(AuthDialog):
    def __init__(self, event, auth):

        super().__init__()

        self.event = event
        self.authtype = 'basic'

        authuser = auth.get('user', '')
        authpass = auth.get('password', '')

        self.authuser.text = authuser
        self.authuser.buffer.document = Document(authuser, len(authuser))

        self.authpass_one.text = authpass
        self.authpass_one.buffer.document = Document(authpass, len(authpass))

        self.authpass_two.text = authpass
        self.authpass_two.buffer.document = Document(authpass, len(authpass))

        self.dialog = Dialog(
            title='Basic Authentication',
            body=HSplit([
                Label(text='Username:\n'),
                self.authuser,
                Window(height=1, char=' '),
                Label(text='Password'),
                self.authpass_one,
                Window(height=1, char=' '),
                Label(text='Retype password'),
                self.authpass_two
            ]),
            buttons=[self.ok_button, self.cancel_button, self.delete_button],
            width=D(preferred=80),
            with_background=True,
            modal=True)

        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)


class DigestAuthDialog(AuthDialog):
    def __init__(self, event, auth):

        super().__init__()

        self.event = event
        self.authtype = 'digest'

        authuser = auth.get('user', '')
        authpass = auth.get('password', '')

        self.authuser.text = authuser
        self.authuser.buffer.document = Document(authuser, len(authuser))

        self.authpass_one.text = authpass
        self.authpass_one.buffer.document = Document(authpass, len(authpass))

        self.authpass_two.text = authpass
        self.authpass_two.buffer.document = Document(authpass, len(authpass))

        self.dialog = Dialog(
            title='Digest Authentication',
            body=HSplit([
                Label(text='Username:\n'),
                self.authuser,
                Window(height=1, char=' '),
                Label(text='Password'),
                self.authpass_one,
                Window(height=1, char=' '),
                Label(text='Retype password'),
                self.authpass_two
            ]),
            buttons=[self.ok_button, self.cancel_button, self.delete_button],
            width=D(preferred=80),
            with_background=True,
            modal=True)

        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)



import tempfile
from subprocess import call
import os

from freud.utils import select_item
from freud.ui.text_buffers import response_buffer, header_buffer
from freud.ui.dialog import ErrorDialog
from freud.model import db
from freud.utils import ButtonManager


def update_body(event):
    """Uses external editor to create/update request body."""

    EDITOR = os.environ.get('EDITOR', None)

    if not EDITOR:
        return ErrorDialog(
            event, title='Error',
            text='Please set your $EDITOR environement variable'
        )

    tf = tempfile.NamedTemporaryFile()

    name = ButtonManager.current_button

    result = db.fetch_one(name=name)

    if result.body:
        with open(tf.name, 'w') as fout:
            fout.write(result.body)

    j = "+'set ft=json"

    call([EDITOR, j, tf.name])

    with open(tf.name, 'r') as fin:
        body = fin.read()

    db.update_one(values={'name': name,
                          'body': body})

    event.app.reset()
    select_item(event)


def open_response_in_editor(event):

    EDITOR = os.environ.get('EDITOR', None)

    if not EDITOR:
        return ErrorDialog(
            event, title='Error',
            text='Please set your $EDITOR environement variable'
        )

    tf = tempfile.NamedTemporaryFile()

    headers = header_buffer.text
    response = response_buffer.text

    set_filetype = None
    if 'content-type: application/json' in headers.lower():
        set_filetype = 'json'
    elif 'text/html' in headers.lower():
        set_filetype = 'html'

    with open(tf.name, 'w') as fout:
        fout.write(str(response))

    if set_filetype:
        call([EDITOR, '+set ft={}'.format(set_filetype), tf.name])
    else:
        call([EDITOR, tf.name])

    event.app.reset()



from prompt_toolkit.buffer import Buffer
from prompt_toolkit.layout.controls import BufferControl
from prompt_toolkit.lexers import PygmentsLexer
from prompt_toolkit.layout.margins import ScrollbarMargin, NumberedMargin
from prompt_toolkit.layout.containers import Window
from pygments.lexers import JsonLexer
from prompt_toolkit.layout import HSplit

from prompt_toolkit.widgets import SearchToolbar

from pygments.lexer import RegexLexer, bygroups
from pygments.token import Text, Keyword, String

from freud import HEADER_HEIGHT, SUMMARY_HEIGHT

response_buffer = Buffer(
    read_only=True, enable_history_search=True, name='response_buffer')
header_buffer = Buffer(read_only=True, name='header_buffer')
summary_buffer = Buffer(read_only=True, name='summary_buffer')

search_toolbar = SearchToolbar()


class HeaderLexer(RegexLexer):
    tokens = {
        'root': [
            (r'(\[)(\')(\d+.*?)(\')(\] )(\| )(\d+.*?\n)',
                bygroups(Keyword, None, String, None, Keyword, None, Text)),
            (r'(\S+:\s)(.*?$)',
                bygroups(String, Text))
        ]
    }


class ResponseLexer(JsonLexer):
    tokens = {
        'root': [
            (r'(.*?\: )(.*?[\[|\{|\n])',
                bygroups(String, Text)),
            (r'(\s*?[\}|\]])',
                bygroups(Text)),
        ]
    }


class ResponseBox:
    def create(self):
        from freud.key_bindings import response_kb, header_kb

        right_margins = [ScrollbarMargin(display_arrows=True)]
        left_margins = [NumberedMargin()]

        self.buffer_control = BufferControl(
            lexer=PygmentsLexer(JsonLexer),
            search_buffer_control=search_toolbar.control,
            buffer=response_buffer)

        t = Window(
            wrap_lines=True,
            right_margins=right_margins,
            left_margins=left_margins,
            height=HEADER_HEIGHT,
            content=BufferControl(
                key_bindings=header_kb,
                search_buffer_control=search_toolbar.control,
                lexer=PygmentsLexer(HeaderLexer), buffer=header_buffer)
        )

        w = Window(
            left_margins=left_margins,
            right_margins=right_margins,
            wrap_lines=True,
            content=self.buffer_control
        )

        return HSplit([
            t,
            Window(height=1, char='─', style='class:line'),
            w,
        ], key_bindings=response_kb)


response_box = ResponseBox()


def summary_box():

    from freud.key_bindings import summary_kb

    return Window(
        content=BufferControl(
            key_bindings=summary_kb,
            lexer=PygmentsLexer(JsonLexer), buffer=summary_buffer
        ), height=SUMMARY_HEIGHT
    )



from prompt_toolkit.layout.dimension import D
from prompt_toolkit import HTML
from prompt_toolkit.widgets import Button, Dialog, Label
from prompt_toolkit.layout.containers import (
    VSplit, HSplit, Window, HorizontalAlign)
from prompt_toolkit.layout import Float

from freud.utils import ButtonManager
from freud.ui.root_container import root_container
from freud.ui.server_container import servers
from freud import KEYS


class KeyQuickRef:

    """ Displays a quick reference for key bindings """

    def __init__(self, event):

        def ok_handler():

            root_container.floats.pop()
            if ButtonManager.prev_button:
                event.app.layout.focus(ButtonManager.prev_button)
            else:
                event.app.layout.focus(servers.content)

        ok_button = Button(text='OK', handler=ok_handler)

        # Create dialog with title
        self.input = HSplit([
            Label(text=HTML(
                'Default Vi key bindings.')
            ),
            Window(height=1, char=' ')
        ])

        # Add navigation keys to dialog
        self.input.children.append(VSplit([
            Label(text=HTML('<b>Basic navigation</b>\n'))])
        )
        for key in navigation_keys:
            new_key = KeyFactory(
                key=key, value=navigation_keys[key])
            self.input.children.append(new_key.vsplit)

        spacer = Window(height=1, char='─')
        blank_spacer = Window(height=1, char=' ')

        self.input.children.append(blank_spacer)
        self.input.children.append(spacer)

        # Add server keys to dialog
        self.input.children.append(VSplit([
            Label(text=HTML('<b>Server keys</b>\n'))])
        )
        for key in server_keys:
            new_key = KeyFactory(
                key=key, value=server_keys[key])
            self.input.children.append(new_key.vsplit)

        self.input.children.append(blank_spacer)
        self.input.children.append(spacer)

        # Add response keys to dialog
        self.input.children.append(VSplit([
            Label(
                text=HTML(
                    '<b>Text areas (response, headers, summary)</b>\n'))])
        )
        for key in response_body_keys:
            new_key = KeyFactory(
                key=key, value=response_body_keys[key])
            self.input.children.append(new_key.vsplit)
        self.input.children.append(blank_spacer)
        self.input.children.append(blank_spacer)

        self.dialog = Dialog(
            title='Key Quick Reference',
            body=self.input,
            buttons=[ok_button],
            width=D(preferred=80),
            with_background=True,
            modal=True)

        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)


class KeyFactory:

    def __init__(self, key=None, value=None):

        self.key = Label(
            text=key,
            width=20
        )
        self.value = Label(
            text=value,
        )
        self.vsplit = VSplit([
            self.key,
            Window(width=1, char=' '),
            self.value
        ], align=HorizontalAlign.LEFT)


# leader = KEYS['leader']

navigation_keys = {
    'tab': 'Switch windows',
    'j/down': 'Next server',
    'k/up': 'Previous server',
    'g': 'First server',
    'G': 'Last server',
}

server_keys = {
    KEYS['send_request']: 'Send request',
    KEYS['new_server']: 'New server',
    KEYS['edit_server']: 'Edit server',
    KEYS['edit_headers']: 'Edit headers',
    KEYS['edit_authentication']: 'Edit auth',
    KEYS['edit_body']: 'Edit body',
    KEYS['delete_server']: 'Delete server',
}

response_body_keys = {
    '/': 'Forward search',
    '?': 'Backward search',
    'n/N': 'Next/Previous search term',
    'Vi Navigation': 'e.g., gg, G, ctrl+u, ctrl+d, etc.',
}



import pytest
import json

from freud.api.request import request_handler
from freud.ui.text_buffers import response_box


class TestResponses:

    @pytest.mark.asyncio
    async def test_get_request(self, db_request_data):
        # Need to create response_box here, so response_box.lexer is available
        response_box.create()

        response = await request_handler('httpbin-get')
        headers, _ = response.get('response')

        assert headers.startswith("['200 Ok']")

    @pytest.mark.asyncio
    async def test_delete_request(self, db_request_data):

        response = await request_handler('httpbin-delete')
        headers, _ = response.get('response')

        assert headers.startswith("['200 Ok']")

    @pytest.mark.asyncio
    async def test_patch_request(self, db_request_data):

        response = await request_handler('httpbin-patch')
        headers, _ = response.get('response')

        assert headers.startswith("['200 Ok']")

    @pytest.mark.asyncio
    async def test_post_request(self, db_request_data):

        response = await request_handler('httpbin-post')
        _, output = response.get('response')

        r = json.loads(output)
        post_data = r['data']

        assert post_data == '{"name": "alice"}'

    @pytest.mark.asyncio
    async def test_put_request(self, db_request_data):

        response = await request_handler('httpbin-put')
        headers, _ = response.get('response')

        assert headers.startswith("['200 Ok']")

    @pytest.mark.asyncio
    async def test_post_form(self, db_request_data):

        response = await request_handler('httpbin-post-form')
        _, output = response.get('response')

        r = json.loads(output)
        form_data = r['form']

        assert form_data['email'] == 'alice@email.com'
        assert form_data['name'] == 'alice'

    @pytest.mark.asyncio
    async def test_basic_auth(self, db_request_data):

        response = await request_handler('httpbin-basic-auth-200')
        headers, _ = response.get('response')

        assert headers.startswith("['200 Ok']")

        response = await request_handler('httpbin-basic-auth-401')
        headers, _ = response.get('response')

        assert headers.startswith("['401 Unauthorized']")

    @pytest.mark.asyncio
    async def test_digest_auth(self, db_request_data):

        response = await request_handler('httpbin-digest-auth-200')
        headers, _ = response.get('response')

        assert headers.startswith("['200 Ok']")

        response = await request_handler('httpbin-digest-auth-401')
        headers, _ = response.get('response')

        assert headers.startswith("['401 Unauthorized']")



from prompt_toolkit.styles import Style
from prompt_toolkit.styles.pygments import style_from_pygments_cls
from pygments.styles import get_style_by_name

from freud import STYLE

theme = STYLE['theme']
line_fg = STYLE['separator_line_fg']
line_bg = STYLE['separator_line_bg']


custom_style = [
    ('line', 'bg:{} fg:{}'.format(line_bg, line_fg)),
    ('button.selected', 'bg:#cccccc fg:#880000'),
]

style = style_from_pygments_cls(get_style_by_name(theme))

style._style_rules.extend(custom_style)



class TestCRUD:
    def test_add_one(self, db):
        result = db.add_one(
            {'name': 'alice', 'url': 'alice.com', 'method': 'get'})

        errors = result.get('errors')
        success = result.get('success')

        assert errors is None
        assert success is True

    def test_fetch_one(self, db, db_dummy_data):
        result = db.fetch_one(name='alice')

        assert result.name == 'alice'
        assert result.url == 'alice.com'
        assert result.method == 'get'

    def test_update_one(self, db, db_dummy_data):
        result = db.fetch_one(name='alice')

        db.update_one(rowid=result.rowid, values={
            'name': 'eve', 'method': 'post', 'url': 'example.com'})

        result = db.fetch_one(name='eve')

        assert result.name == 'eve'
        assert result.method == 'post'

    def test_fetch_all(self, db, db_dummy_data):
        rows = db.fetch_all()

        assert len(rows) == 2

    def test_delete_one(self, db, db_dummy_data):
        before_delete = db.fetch_all()

        db.delete_one(name='alice')

        after_delete = db.fetch_all()

        assert len(before_delete) == len(after_delete) + 1


class TestErrors:

    def test_add_one_missing_column(self, db):
        result = db.add_one({})

        errors = result.get('errors')

        assert errors == 'missing column error: name'

        result = db.add_one(
            {'name': 'alice'})

        errors = result.get('errors')

        assert errors == 'missing column error: url'

        result = db.add_one(
            {'name': 'alice', 'url': 'alice.com'})

        errors = result.get('errors')

        assert errors == 'missing column error: method'

        result = db.add_one(
            {'name': 'alice', 'url': 'alice.com', 'method': 'get'})

        errors = result.get('errors')

        assert errors is None

    def test_constraint_violation(self, db, db_dummy_data):
        result = db.add_one(
            {'name': 'alice', 'url': 'alice.com', 'method': 'get'})

        errors = result.get('errors')

        assert errors == '{}'.format(
            'sqlite error: UNIQUE constraint failed: requests.name')


class TestSort:

    def test_sort(self, db, db_dummy_data):

        name_asc = db.fetch_all(sort_by='name', order='asc')
        assert name_asc[0].name == 'alice'
        assert name_asc[1].name == 'bob'

        name_desc = db.fetch_all(sort_by='name', order='desc')
        assert name_desc[0].name == 'bob'
        assert name_desc[1].name == 'alice'

        time_asc = db.fetch_all(sort_by='timestamp', order='asc')
        assert time_asc[0].name == 'alice'
        assert time_asc[1].name == 'bob'

        time_desc = db.fetch_all(sort_by='timestamp', order='desc')
        assert time_desc[0].name == 'bob'
        assert time_desc[1].name == 'alice'




import configparser
import os
import sys
import pkg_resources

folder = 'config/'

config_file = os.path.join(folder, 'freud.ini')

__version__ = pkg_resources.get_distribution(__package__).version

DEFAULT_SETTINGS = '''
[LAYOUT]
# Width of sidebar that contains server names
server_width = 15
# Height of container that holds headers
header_height = 10
# Height of container that shows server summary
summary_height = 10

[KEYS]
new_server = n
edit_server = e
send_request = r
edit_authentication = a
edit_headers = h
edit_body = b
delete_server = d
open_response_body = o
sort_servers = s
key_quick_ref = c-f

[DB]
filename = requests.db

[JSON]
indentation = 2

[SORT_BY]
# Column options: name, timestamp, url, method, body, authtype, authuser,
# authpass, headers
# Order options: asc, desc
column = timestamp
order = asc

[STYLE]
# More styles here:
# https://bitbucket.org/birkenfeld/pygments-main/src/stable/pygments/styles
theme = default
separator_line_fg = gray
separator_line_bg = black
'''

if not os.path.exists(folder):
    os.makedirs(folder)

if not os.path.exists(config_file):
    with open(config_file, 'w') as cfile:
        cfile.write(DEFAULT_SETTINGS)

config = configparser.ConfigParser()

config.read(config_file)

if hasattr(sys, '_called_from_test'):
    config['DB']['filename'] = 'delete_freud_test_database.db'


KEYS = config['KEYS']

SERVER_WIDTH = int(config['LAYOUT']['server_width'])

HEADER_HEIGHT = int(config['LAYOUT']['header_height'])

SUMMARY_HEIGHT = int(config['LAYOUT']['summary_height'])

STYLE = config['STYLE']

DB_FILE = config['DB']['filename']

SORT_BY = config['SORT_BY']

INDENTATION = int(config['JSON']['indentation'])



from prompt_toolkit.layout.containers import HSplit
from prompt_toolkit.layout import FloatContainer
from prompt_toolkit.layout import Float
from prompt_toolkit.layout.menus import CompletionsMenu

from freud.ui.body_container import titled_body
from freud.utils import SortOrder


class RootContainer:

    def create(self):

        sort_by = SortOrder.sort_by
        order = SortOrder.order

        self.body = titled_body.create(sort_by=sort_by, order=order)

        self.container = HSplit([self.body])

        completions = Float(xcursor=True,
                            ycursor=True,
                            content=CompletionsMenu(max_height=16, scroll_offset=1))

        self.float_container = FloatContainer(
            content=self.container, floats=[completions])

        self.floats = self.float_container.floats

        return self.float_container


root_container = RootContainer()



import requests
import json

from freud.model import db
from freud.api.response import response_handler


def session_builder(req):
    s = requests.Session()
    request = requests.Request(req.method, req.url)

    headers = {
        'user-agent': 'Freud',
        'accept-encoding': 'gzip, deflate',
        'accept': '*/*',
        'connection': 'keep-alive',
        'charset': 'UTF-8',
    }
    request.headers = headers

    if req.auth:

        auth = json.loads(req.auth)
        authtype = auth.get('type')
        authuser = auth.get('user')
        authpass = auth.get('password')

        if authtype == 'basic':
            request.auth = (authuser, authpass)
        elif authtype == 'digest':
            request.auth = requests.auth.HTTPDigestAuth(
                authuser, authpass)

    if req.headers:
        headers = json.loads(req.headers)
        headers = {k.lower(): v.lower() for k, v in headers.items()}
        request.headers.update(headers)

    content_type = request.headers.get('content-type', None)

    if content_type:
        request.content_type = content_type

    request.data = req.body

    response = None
    errors = None

    try:
        response = s.send(request.prepare())

    except requests.exceptions.ConnectionError as e:
        errors = {'Connection error': str(e)}

    except requests.exceptions.RequestException as e:
        errors = {'Request Exception': str(e)}

    return [response, errors]


async def request_handler(name):

    # Get selected server url
    server = db.fetch_one(name=name)

    response, errors = session_builder(server)

    if errors:
        return {'errors': errors}

    headers, response_body = response_handler(response)

    return {'response': [headers, response_body]}



from prompt_toolkit.layout.containers import (
    VSplit, HSplit, Window, WindowAlign)
from prompt_toolkit.layout.controls import FormattedTextControl

from freud.ui.text_buffers import response_box, summary_box, search_toolbar
from freud.ui.server_container import servers
from freud import __version__


class TitledBody:

    def create(self, sort_by=None, order=None):

        self.servers = servers.refresh(sort_by=sort_by, order=order)

        servers_output = VSplit([
            self.servers,
            Window(width=1, char='│', style='class:line'),
            response_box.create(),
        ])

        body = HSplit([
            servers_output,
            Window(height=1, char='─', style='class:line'),
            summary_box()
        ])

        title_bar = [
            ('class:title', ' Freud {}'.format(__version__)),
            ('class:title',
             ' (Press [Ctrl-C] to quit. Press [Ctrl-F] for info.)'),
        ]

        self.container = HSplit([
            search_toolbar,
            Window(height=1,
                   content=FormattedTextControl(title_bar),
                   align=WindowAlign.CENTER),
            Window(height=1, char='─', style='class:line'),
            body,
        ])

        return self.container


titled_body = TitledBody()



import json

from prompt_toolkit.layout.dimension import D
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.widgets import Button, TextArea, Dialog, Label
from prompt_toolkit.layout.containers import VSplit, HSplit, Window
from prompt_toolkit.layout import Float
from prompt_toolkit.key_binding import KeyBindings


from freud.ui.dialog import ErrorDialog
from freud.ui.root_container import root_container
from freud.model import db
from freud.utils import ButtonManager, select_item


headers = [
    'Accept',
    'Accept-Charset',
    'Accept-Encoding',
    'Accept-Language',
    'Accept-Datetime',
    'Authorization',
    'Cache-Control',
    'Connection',
    'Cookie',
    'Content-Length',
    'Content-MD5',
    'Content-Type',
    'Date',
    'Expect',
    'Forwarded',
    'From',
    'Host',
    'If-Match',
    'If-Modified-Since',
    'If-None-Match',
    'If-Range',
    'If-Unmodified-Since',
    'Max-Forwards',
    'Origin',
    'Pragma',
    'Proxy-Authorization',
    'Range',
    'Referer',
    'TE',
    'User-Agent',
    'Upgrade',
    'Via',
    'Warning',
]

header_values = [
    'application/json',
    'text/html'
]

headers_completer = WordCompleter(headers,
                                  ignore_case=True)

header_values_completer = WordCompleter(header_values,
                                        ignore_case=True)


class HeadersDialog:
    def __init__(self, event):

        def ok_handler():
            result = add_headers_to_db(event, name)

            if result.get('success'):
                root_container.floats.pop()
                event.app.layout.focus(ButtonManager.prev_button)
                select_item(event)

        def cancel_handler():
            root_container.floats.pop()
            event.app.layout.focus(ButtonManager.prev_button)

        # Get data about server currently editing
        name = ButtonManager.current_button
        result = db.fetch_one(name=name)

        # Dialog configuration
        ok_button = Button(text='OK', handler=ok_handler)
        cancel_button = Button(text='Cancel', handler=cancel_handler)

        local_kb = KeyBindings()

        @local_kb.add('enter')
        def cancel_completion(event):
            buff = event.app.current_buffer
            buff.complete_state = None
            event.app.layout.focus_next()

        @local_kb.add('c-n')
        def add_header(event):
            new_header = HeaderFactory()
            spacer = Window(height=1, char=' ')
            self.input.children.append(spacer)
            self.input.children.append(new_header.vsplit)

        # Setup initial headings for headers
        self.input = HSplit([
            Label(text='Press ctrl+n to add another header'),
            Window(height=1, char='─'),
            VSplit([
                Label(text='Header'),
                Window(width=1, char=' '),
                Label(text='Type'),
            ]),
            Window(height=1, char=' ')
        ], key_bindings=local_kb)

        # If there are existing headers, include them in the layout
        if result.headers:
            headers = json.loads(result.headers)
            for header in headers:
                new_header = HeaderFactory(
                    header=header, value=headers[header])
                spacer = Window(height=1, char=' ')
                self.input.children.append(spacer)
                self.input.children.append(new_header.vsplit)

        # If there are no headers currently assigned to the server, create
        # empty boxes for input
        else:
            new_header = HeaderFactory()
            spacer = Window(height=1, char=' ')
            self.input.children.append(spacer)
            self.input.children.append(new_header.vsplit)

        self.dialog = Dialog(
            title='Headers',
            body=self.input,
            buttons=[ok_button, cancel_button],
            width=D(preferred=80),
            with_background=True
        )

        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)


class HeaderFactory:

    headers = []

    def __init__(self, header=None, value=None):

        header = header if header else ''
        value = value if value else ''

        self.header = TextArea(
            multiline=False,
            password=False,
            completer=headers_completer,
            text=header
        )
        self.value = TextArea(
            multiline=False,
            password=False,
            completer=header_values_completer,
            text=value
        )
        self.vsplit = VSplit([
            self.header,
            Window(width=1, char=' '),
            self.value,
        ])
        HeaderFactory.headers.append(self)


def add_headers_to_db(event, name):

    headers = {}
    for header in HeaderFactory.headers:
        if header.header.text == '' and header.value.text == '':
            # Skip empty input boxes
            continue

        if headers.get(header.header.text, None):

            ErrorDialog(event, title='Duplicate Header',
                        text='Please remove duplicate header')

            return {'success': False}

        headers[header.header.text] = header.value.text

    if len(headers) == 0:
        # Since there are no headers, set headers to None in db
        result = db.update_one(values={'name': name,
                                       'headers': None})
    else:
        result = db.update_one(values={'name': name,
                                       'headers': json.dumps(headers)})

    # Delete all headers from factory
    HeaderFactory.headers = []

    if result.get('errors'):
        ErrorDialog(event, title='Header error',
                    text=result.get('errors'))

        return {'success': False}

    return {'success': True}



import sqlite3
import os
import datetime
from collections import namedtuple

from freud import DB_FILE, SORT_BY

basedir = os.path.abspath(os.path.dirname(__file__))


class Db:

    def __init__(self, database=DB_FILE):

        self.database = database
        self.connect()
        self.close()

    def namedtuple_factory(self, cursor, row):
        """ Returns namedtuple results, named with column names """

        fields = [col[0] for col in cursor.description]
        Row = namedtuple('Row', fields)
        return Row(*row)

    def connect(self):

        self.conn = sqlite3.connect(
            os.path.join(basedir, self.database),
            detect_types=sqlite3.PARSE_COLNAMES
        )
        self.conn.row_factory = self.namedtuple_factory
        self.cursor = self.conn.cursor()

        self.cursor.execute('''CREATE TABLE IF NOT EXISTS requests (
                            name text not null unique,
                            timestamp timestamp,
                            url text not null,
                            method text not null,
                            body text,
                            auth text,
                            headers text
                            )
                    ''')

    def close(self):

        self.conn.commit()
        self.conn.close()

    def fetch_all(self, sort_by=None, order=None):
        """ Returns all rows """

        sort_by = sort_by if sort_by else SORT_BY['column']
        order = order if order else SORT_BY['order']

        self.connect()

        if sort_by and order:
            self.cursor.execute(
                'SELECT * FROM requests ORDER BY {} {}'.format(sort_by, order))
        elif sort_by:
            self.cursor.execute(
                'SELECT * FROM requests ORDER BY {sort_by}'.format(sort_by))
        else:
            self.cursor.execute('SELECT * FROM requests')

        rows = self.cursor.fetchall()

        self.close()

        return rows

    def fetch_one(self, rowid=None, name=None):

        self.connect()

        if rowid:
            self.cursor.execute('''SELECT * FROM requests WHERE rowid = ?''',
                                (rowid,))
        else:
            self.cursor.execute('''
                    SELECT rowid, * FROM requests WHERE name = ?''',
                                (name,))

        result = self.cursor.fetchone()

        if result:
            return result

        return False

        self.close()

    def add_one(self, values):

        self.connect()

        try:
            with self.conn:
                self.conn.execute('''INSERT INTO requests
                                 (
                                  name, url, method, timestamp, auth,
                                  body, headers)
                                  VALUES (?, ?, ?, ?, ?, ?, ?)''',
                                  (
                                      values['name'],
                                      values['url'],
                                      values['method'],
                                      datetime.datetime.now(),
                                      values.get('auth'),
                                      values.get('body'),
                                      values.get('headers')
                                  ))
        except sqlite3.IntegrityError as e:
            return {'errors': 'sqlite error: {}'.format(e.args[0])}

        except KeyError as e:
            return {'errors': 'missing column error: {}'.format(e.args[0])}

        finally:
            self.close()

        return {'success': True}

    def delete_one(self, name):

        self.connect()
        self.cursor.execute('''DELETE FROM requests WHERE name = ?''',
                            (name,))

        self.close()

    def delete_all(self):
        """ Used for testing """

        self.connect()
        self.cursor.execute('''DELETE FROM requests''')

        self.close()

    def update_one(self, values=None, rowid=None):
        """ Updates one row by rowid if supplied, else by name """

        self.connect()

        if rowid:
            # If the rowid is set, we could be changing the name
            name = values.get('name')
            url = values.get('url')
            method = values.get('method')

            if not all([name, url, method]):
                return {'errors': 'Name, url, and method are required'}

            try:
                with self.conn:
                    self.conn.execute('''UPDATE requests SET
                            url=?, method=?, name=? where rowid=?''',
                                      (url, method, name, rowid))

            except sqlite3.IntegrityError as e:
                return {'errors': 'sqlite error: {}'.format(e.args[0])}

            except KeyError as e:
                return {
                    'errors':  'missing column error: {}'.format(e.args[0])}

            finally:
                self.close()

        else:
            # Changes all fields passed in through values
            name = values.get('name')

            result = self.fetch_one(name=name)

            body = values.get('body', result.body)
            headers = values.get('headers', result.headers)
            auth = values.get('auth', result.auth)

            self.connect()

            try:

                with self.conn:
                    self.conn.execute('''
                            UPDATE requests SET
                            body=?, headers=?, auth=?
                            where name=?''',
                                      (body, headers, auth,
                                       name
                                       )
                                      )

            except sqlite3.IntegrityError as e:
                return {'errors': 'sqlite error: {}'.format(e.args[0])}

            except KeyError as e:
                return {'errors': 'missing column error: {}'.format(e.args[0])}

            finally:
                self.close()

        return {'success': True}


db = Db()



from prompt_toolkit.layout import Float
from prompt_toolkit.layout.dimension import D
from prompt_toolkit.widgets import Button, Dialog, Label
from prompt_toolkit.layout.containers import Window
from prompt_toolkit.layout.controls import FormattedTextControl
from prompt_toolkit.filters import to_filter

from freud.model import db
from freud.ui.root_container import root_container
from freud.utils import ButtonManager, select_item
from freud.ui.server_container import servers
from freud.ui.text_buffers import summary_buffer


class DeleteDialog:
    def __init__(self, event):

        def ok_handler():
            # if len(ButtonManager.buttons) > 0:
            delete_server(event, name)
            root_container.floats.pop()

        def cancel_handler():
            root_container.floats.pop()
            event.app.layout.focus(ButtonManager.prev_button)

        # Get data about server currently editing
        name = ButtonManager.current_button

        # Dialog configuration
        ok_button = Button(text='OK', handler=ok_handler)
        cancel_button = Button(text='Cancel', handler=cancel_handler)

        self.dialog = Dialog(
            title='Delete confirmation',
            body=Label(
                text='Are you sure you want to delete {}?'.format(name)),
            buttons=[cancel_button, ok_button],
            width=D(preferred=80),
            with_background=True
        )

        root_container.floats.append(Float(self.dialog))
        event.app.layout.focus(self.dialog)


def delete_server(event, name):

    db.delete_one(name)

    buttons = servers.hsplit.children

    # The server has been removed from the db. Now, we have to remove the
    # button from the layout by iterating over buttons and removing it by index
    for idx, button in enumerate(buttons):
        if name == button.content.text()[1][1].strip():

            del buttons[idx]

            if len(buttons) > 0:

                try:
                    # If the deleted button was not the last button
                    event.app.layout.focus(buttons[idx])
                    ButtonManager.prev_button = buttons[idx]
                    select_item(event)

                except IndexError:
                    # If the deleted button was the last button
                    event.app.layout.focus(buttons[idx - 1])
                    ButtonManager.prev_button = buttons[idx - 1]
                    select_item(event)

            else:
                # Last button was deleted, display message "No servers"

                control = FormattedTextControl(
                    'No Servers',
                    focusable=True,
                    show_cursor=False)

                window = Window(
                    control,
                    height=1,
                    dont_extend_width=True,
                    dont_extend_height=True)

                buttons.append(window)

                summary_buffer.read_only = to_filter(False)
                summary_buffer.text = ''
                summary_buffer.read_only = to_filter(True)

                ButtonManager.prev_button = None
                ButtonManager.current_button = None

                event.app.layout.focus(servers.content)



import asyncio

from prompt_toolkit.application import Application
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit.enums import EditingMode
from prompt_toolkit.eventloop import use_asyncio_event_loop

from freud.key_bindings import kb
from freud.ui.server_container import servers
from freud.ui.root_container import root_container
from freud.ui.style import style
from freud.utils import on_startup


use_asyncio_event_loop()


def main():
    app = Application(
        layout=Layout(root_container.create(),
                      focused_element=servers.content),
        key_bindings=kb,
        editing_mode=EditingMode.VI,
        style=style,
        mouse_support=True,
        full_screen=True,
        after_render=on_startup
    )

    asyncio.get_event_loop().run_until_complete(
        app.run_async().to_asyncio_future()
    )



from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.filters import Condition

from freud.server_control.delete_server import DeleteDialog
from freud.server_control.headers_dialog import HeadersDialog
from freud.server_control.server_dialog import ServerDialog
from freud.utils import ButtonManager
from freud.ui.text_buffers import (
    response_buffer, header_buffer, summary_buffer)
from freud.server_control.call_editor import (
    update_body, open_response_in_editor)
from freud.server_control.auth_dialog import AuthSelector
from freud.ui.keys import KeyQuickRef
from freud.ui.sort import SortDialog
from freud.api import send_request
from freud import KEYS


kb = KeyBindings()
server_kb = KeyBindings()
response_kb = KeyBindings()
header_kb = KeyBindings()
summary_kb = KeyBindings()
completion = KeyBindings()


@Condition
def is_button():
    """ Filter when there are no buttons """

    return len(ButtonManager.buttons) > 0


# Global Keys
#

@kb.add('c-c', eager=True)
def quit_app(event):
    event.app.exit()


@server_kb.add('c-f')
@response_kb.add('c-f')
def keys_quick_reference(event):
    KeyQuickRef(event)


# Navigation Keys
#

@server_kb.add('tab', filter=is_button)
def go_to_header_from_server(event):
    event.app.layout.focus(header_buffer)


@server_kb.add('s-tab', filter=is_button)
def go_to_summary_from_servers(event):
    event.app.layout.focus(summary_buffer)


@header_kb.add('tab', filter=is_button)
def go_to_response_from_header(event):
    event.app.layout.focus(response_buffer)


@header_kb.add('s-tab', filter=is_button)
def go_to_server_from_header(event):
    event.app.layout.focus(ButtonManager.prev_button)


@response_kb.add('tab', filter=is_button)
def go_to_summary_from_response(event):
    event.app.layout.focus(summary_buffer)


@response_kb.add('s-tab', filter=is_button)
def go_to_headers_from_resposne(event):
    event.app.layout.focus(header_buffer)


@summary_kb.add('tab')
def go_to_servers_from_summary(event, filter=is_button):
    event.app.layout.focus(ButtonManager.prev_button)


@summary_kb.add('s-tab', filter=is_button)
def go_to_response_from_summary(event):
    event.app.layout.focus(response_buffer)


@response_kb.add('h', filter=is_button)
@response_kb.add('left', filter=is_button)
def go_to_servers(event):
    if (len(ButtonManager.buttons) > 0):
        event.app.layout.focus(ButtonManager.prev_button)


@server_kb.add('l', filter=is_button)
@server_kb.add('right', filter=is_button)
def go_to_response(event):
    event.app.layout.focus(response_buffer)


@server_kb.add('g', filter=is_button)
def top_of_list(event):
    """ Adds vi binding to jump to top of server list """

    buttons = ButtonManager.buttons

    event.app.layout.focus(buttons[0])
    ButtonManager.prev_button = buttons[0]


@server_kb.add('G', filter=is_button)
def bottom_of_list(event):
    """ Adds vi binding to jump to bottom of server list """

    buttons = ButtonManager.buttons

    event.app.layout.focus(buttons[-1])
    ButtonManager.prev_button = buttons[-1]


@server_kb.add('j', filter=is_button)
@server_kb.add('down')
def down_button(event):
    """ Scroll down list of servers in the server_container """

    buttons = ButtonManager.buttons

    current_window = event.app.layout.current_window
    idx = buttons.index(current_window)

    try:
        event.app.layout.focus(buttons[idx + 1])
        ButtonManager.prev_button = buttons[idx + 1]
    except IndexError:
        # If we're at the button item, loop to first item

        event.app.layout.focus(buttons[0])
        ButtonManager.prev_button = buttons[0]


@server_kb.add('k', filter=is_button)
@server_kb.add('up')
def up_button(event):
    """ Scroll up list of servers in the server_container """

    buttons = ButtonManager.buttons

    current_window = event.app.layout.current_window
    idx = buttons.index(current_window)

    try:
        event.app.layout.focus(buttons[idx - 1])
        ButtonManager.prev_button = buttons[idx - 1]
    except IndexError:
        # If we're at the top item, loop to last item

        event.app.layout.focus(buttons[-1])
        ButtonManager.prev_button = buttons[-1]


# Server control keys
#
@server_kb.add(KEYS['open_response_body'], filter=is_button)
@response_kb.add(KEYS['open_response_body'], filter=is_button)
def open_response(event):
    open_response_in_editor(event)


@server_kb.add(KEYS['sort_servers'], filter=is_button)
def sort(event):
    SortDialog(event)


@server_kb.add(KEYS['new_server'])
def new_server(event):
    ServerDialog(event, create_server=True)


@server_kb.add(KEYS['edit_headers'], filter=is_button)
def new_headers(event):
    HeadersDialog(event)


@server_kb.add(KEYS['edit_server'], filter=is_button)
def edit(event):
    ServerDialog(event)


@server_kb.add(KEYS['edit_authentication'], filter=is_button)
def edit_authentication(event):
    AuthSelector(event)


@server_kb.add(KEYS['edit_body'], filter=is_button)
def edit_body(event):
    update_body(event)


@server_kb.add(KEYS['delete_server'], filter=is_button)
def rm_server(event):
    DeleteDialog(event)


@server_kb.add(KEYS['send_request'], filter=is_button)
def request(event):
    send_request(event)



from prompt_toolkit.layout import HSplit
from prompt_toolkit.widgets import Box

from freud.utils import CustomButton
from freud.utils import ButtonManager
from freud.model import db
from freud import SERVER_WIDTH


def update_servers(sort_by=None, order=None):

    rows = db.fetch_all(sort_by=sort_by, order=order)
    values = []
    for row in rows:
        values.append(row[0])

    return values


class ServerContainer:

    def refresh(self, sort_by=None, order=None):

        names = update_servers(sort_by=sort_by, order=order)

        self.server_list = []
        for name in names:
            button_obj = ButtonManager(name)
            button = CustomButton(name, handler=button_obj.click_handler)
            self.server_list.append(button)

        # If there are no servers
        if not self.server_list:
            from prompt_toolkit.layout.containers import Window
            from prompt_toolkit.layout.controls import FormattedTextControl

            window = Window(
                FormattedTextControl(
                    'No Servers',
                    focusable=True,
                    show_cursor=False
                )
            )

            self.server_list.append(window)

        self.hsplit = HSplit(self.server_list)

        boxes = Box(
            width=SERVER_WIDTH,
            padding_top=0,
            padding_left=1,
            body=self.hsplit
        )

        from freud.key_bindings import server_kb

        self.content = HSplit([
            boxes
        ], key_bindings=server_kb)

        return self.content


servers = ServerContainer()



from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='freud',
    version='0.1.1',
    author='Stephen Martin',
    author_email='lockwood@opperline.com',
    description='TUI REST client to analyze API endpoints',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/stloma/freud',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='cli tui http rest request api',
    packages=find_packages(),
    install_requires=['requests', 'prompt_toolkit==2.0.3', 'pygments'],
    tests_require=['pytest', 'pytest_asyncio', 'pytest_httpbin'],
    entry_points={
        'console_scripts': [
            'freud=freud.__main__:main',
        ],
    },
    project_urls={
        'Bug Reports': 'https://github.com/stloma/freud/issues',
        'Source': 'https://github.com/stloma/freud',
    },
)



from prompt_toolkit.layout.dimension import D
from prompt_toolkit.widgets import Button
from prompt_toolkit.layout.containers import HSplit, Window
from prompt_toolkit.widgets import Dialog
from prompt_toolkit.layout.containers import WindowAlign
from prompt_toolkit.layout import Float
from prompt_toolkit.layout.controls import FormattedTextControl

from freud.ui.root_container import root_container
from freud.utils import ButtonManager


class ErrorDialog:
    def __init__(self, event, title='', text=''):

        def ok_handler():
            root_container.floats.pop()

            # If there was an original dialog, insert it back into layout
            if self.orig_dialog:
                root_container.floats.append(self.orig_dialog)
                event.app.layout.focus(root_container.float_container)
            else:
                event.app.layout.focus(ButtonManager.prev_button)

        ok_button = Button(text='OK', handler=ok_handler)

        dialog = Dialog(
            Window(
                wrap_lines=True,
                content=FormattedTextControl(text=text),
                always_hide_cursor=True
            ),
            title=title,
            width=D(preferred=80),
            buttons=[ok_button],
            with_background=True
        )

        try:
            # If a dialog was already up, save it
            self.orig_dialog = root_container.floats.pop()
        except IndexError:
            self.orig_dialog = None

        root_container.floats.append(Float(content=dialog))
        event.app.layout.focus(ok_button)


class LoadingDialog:

    """ Displays a loading message when waiting for a server response """

    def __init__(
            self, event=None, title='', text=''):

        content = Window(
            height=3,
            align=WindowAlign.CENTER,
            content=FormattedTextControl(
                text=text,
                show_cursor=False,
                modal=True
            )
        )

        body = HSplit([
            Window(height=1, char=' '),
            content,
            Window(height=1, char=' '),
        ], padding=1)

        dialog = Dialog(
            body,
            title=title,
            width=D(preferred=80),
            with_background=True
        )

        loading_float = Float(content=dialog)

        root_container.floats.append(loading_float)

        self.focus = event.app.layout.focus
        self.focus(content)

    def close_dialog(self):
        root_container.floats.pop()
        self.focus(ButtonManager.prev_button)
